namespace eval dronebl_pub {

# Don't forget to ".chanset #channel +dronebl" on your bot
set chanset_flag "dronebl"

set triggers(add) {
	"!add"
	"!submit"
	"!insert"
}

set triggers(lookup) {
	"!lookup"
	"!query"
	"!search"
	"!list"
	"!find"
}

set triggers(remove) {
	"!remove"
	"!deactivate"
	"!delist"
	"!delete"
	"!unlist"
	"!del"
}

set triggers(classes) {
	"!classes"
	"!categories"
	"!types"
	"!class"
	"!type"
	"!category"
	"!info"
}

set triggers(help) {
	"!help"
	"!dronebl"
}

set triggers(showaccess) {
	"!showaccess"
	"!show"
	"!access"
	"!permissions"
	"!perms"
	"!triggers"
	"!commands"
}

set triggers(stfu) {
	"!stfu"
	"!shutup"
	"!stop"
	"!quit"
	"!die"
	"!shut"
	"!fuck"
	"!quiet"
	"!halt"
}

set permissions(add) m
set permissions(lookup) f
set permissions(remove) m
set permissions(classes) -
set permissions(help) -
set permissions(showaccess) -
set permissions(stfu) f

# end of crap you need to set

if {[catch {set dronebl ::dronebl::key} 0]} {
	putlog "DroneBL library isn't loaded.  Unable to load dronebl-pub.tcl."
	return
}

foreach trigger $triggers(add) {
	bind pub - $trigger ::dronebl_pub::add
}

foreach trigger $triggers(lookup) {
	bind pub - $trigger ::dronebl_pub::lookup
}

foreach trigger $triggers(remove) {
	bind pub - $trigger ::dronebl_pub::remove
}

foreach trigger $triggers(classes) {
	bind pub - $trigger ::dronebl_pub::classes
}

foreach trigger $triggers(help) {
	bind pub - $trigger ::dronebl_pub::help
}

foreach trigger $triggers(showaccess) {
	bind pub - $trigger ::dronebl_pub::showaccess
}

foreach trigger $triggers(stfu) {
	bind pub - $trigger ::dronebl_pub::stfu
}

bind msgm - * ::dronebl_pub::msg_route

setudef flag $chanset_flag

proc display {chan query} {
	set rows [::dronebl::tabulate $query]
	for {set i 0} {$i < [llength $rows]} {incr i} {
		set row [lindex $rows $i]
		if {!$i && [llength $rows] > 1} { set row "\002\037$row\037\002" }
		putserv "PRIVMSG $chan :$row"
	}
	return
}

proc msg_route {nick uhost hand txt} {
	global ::dronebl_pub::triggers
	set words [split $txt]
	set firstword [lindex $words 0]
	set words [lreplace $words 0 0]
	set words [join $words]
	foreach {trigger lst} [array get ::dronebl_pub::triggers] {
		if {[lsearch -exact $lst $firstword] != -1} {
			::dronebl_pub::$trigger $nick $uhost $hand $nick $words
			break
		}
	}
}

proc add {nick uhost hand chan txt} {
	global ::dronebl_pub::chanset_flag ::dronebl_pub::permissions
	if {$chan != $nick && ![channel get $chan $::dronebl_pub::chanset_flag]} {return}
	set p $::dronebl_pub::permissions(add)
	if {![matchattr $hand $p|$p $chan]} {
		puthelp "PRIVMSG $chan :$nick: Permission denied.  If you feel you have reached this message in error, try identing."
		return
	}
	if {$txt == ""} {::dronebl_pub::help $nick $uhost $hand $chan add; return}
	set args [split [string tolower $txt]]

	if {[string is integer -strict [lindex $args 0]]} { set bantype [lindex $args 0] }

	if {[::dronebl::submit $args]} {
		if {[info exists bantype]} {
			set args [lreplace $args 0 0]
		} else {
			set bantype 1
		}
		putserv "PRIVMSG $chan :Success submitting [::dronebl::args2text $args] to DroneBL as type \002$bantype\002."
	} else {
		putserv "PRIVMSG $chan :[::dronebl::lasterror]"
	}
	return
}

proc lookup {nick uhost hand chan txt} {
	global ::dronebl_pub::chanset_flag ::dronebl_pub::permissions
	if {$chan != $nick && ![channel get $chan $::dronebl_pub::chanset_flag]} {return}
	set p $::dronebl_pub::permissions(lookup)
	if {![matchattr $hand $p|$p $chan]} {
		puthelp "PRIVMSG $chan :$nick: Permission denied.  If you feel you have reached this message in error, try identing."
		return
	}
	if {$txt == ""} {::dronebl_pub::help $nick $uhost $hand $chan lookup; return}
	if {[set query [::dronebl::lookup $txt]] == "false"} {
		putserv "PRIVMSG $chan :[::dronebl::lasterror]"
	} else {
		::dronebl_pub::display $chan $query
	}
	return
}

proc remove {nick uhost hand chan txt} {
	global ::dronebl_pub::chanset_flag ::dronebl_pub::permissions
	if {$chan != $nick && ![channel get $chan $::dronebl_pub::chanset_flag]} {return}
	set p $::dronebl_pub::permissions(remove)
	if {![matchattr $hand $p|$p $chan]} {
		puthelp "PRIVMSG $chan :$nick: Permission denied.  If you feel you have reached this message in error, try identing."
		return
	}
	if {$txt == ""} {::dronebl_pub::help $nick $uhost $hand $chan remove; return}
	if {[::dronebl::remove $txt]} {
		set query [::dronebl::lookup $txt]
		::dronebl::display $txt
	} else {
		putserv "PRIVMSG $chan :[::dronebl::lasterror]"
	}
	return
}

proc classes {nick uhost hand chan txt} {
	global ::dronebl_pub::chanset_flag ::dronebl_pub::permissions ::dronebl_pub::triggers
	if {$chan != $nick && ![channel get $chan $::dronebl_pub::chanset_flag]} {return}
	set p $::dronebl_pub::permissions(classes)
	if {![matchattr $hand $p|$p $chan]} {
		puthelp "PRIVMSG $chan :$nick: Permission denied.  If you feel you have reached this message in error, try identing."
		return
	}
	if {[set query [::dronebl::classes $txt]] == "false"} {
		putserv "PRIVMSG $chan :[::dronebl::lasterror]"
	} else {
		::dronebl_pub::display $chan $query
		foreach trigger $triggers(classes) {
			unbind pub - $trigger ::dronebl_pub::classes
			timer 5 "bind pub - $trigger ::dronebl_pub::classes"
		}
	}
	return
}

proc help {nick uhost hand chan txt} {
	global ::dronebl_pub::chanset_flag ::dronebl_pub::permissions ::dronebl_pub::triggers
	if {$chan != $nick && ![channel get $chan $::dronebl_pub::chanset_flag]} {return}
	set p $::dronebl_pub::permissions(help)
	if {![matchattr $hand $p|$p $chan]} {
		puthelp "PRIVMSG $chan :$nick: Permission denied.  If you feel you have reached this message in error, try identing."
		return
	}
	set txt [string tolower [string map {! ""} $txt]]
	set match ""
	foreach {name triggerlist} [array get ::dronebl_pub::triggers] {
		set rxp [string tolower [string map {! ""} "^([join $triggerlist |])$"]]
		if {[regexp $rxp $txt -> match ]} {
			set txt $name
			break
		}
	}
	switch -glob $txt {
		add {
			set p $::dronebl_pub::permissions(add)
			if {![matchattr $hand $p|$p $chan]} {
				puthelp "PRIVMSG $chan :$nick: You don't have access to that command.  If you normally do have access, try identing."
			} else {
				puthelp "PRIVMSG $chan :syntax: \002!$match\002 \[type\] \002ip/host\002 \[ip/host \[ip/host \[ip/host...\]\]\]"
				puthelp "PRIVMSG $chan :...where \[type\] is optional (default: 1)"
			}
		}
		lookup {
			set p $::dronebl_pub::permissions(lookup)
			if {![matchattr $hand $p|$p $chan]} {
				puthelp "PRIVMSG $chan :$nick: You don't have access to that command.  If you normally do have access, try identing."
			} else {
				puthelp "PRIVMSG $chan :syntax: \002!$match\002 \[options\] \002addr/range\002 \[addr/range \[addr/range \[addr/range...\]\]\]"
				puthelp "PRIVMSG $chan :... where the ip can contain *? wildcards and \[1-255\] ranges"
				puthelp "PRIVMSG $chan :... optional switches:"
				puthelp "PRIVMSG $chan :  \002-active\002    -- shows only active listings (default)"
				puthelp "PRIVMSG $chan :  \002-inactive\002  -- shows only listings that have expired or been removed"
				puthelp "PRIVMSG $chan :  \002-all\002       -- shows listings regardless of active status"
				puthelp "PRIVMSG $chan :  \002-limit int\002 -- requests a list of up to \[int\] matches (default: 10)"
			}
		}
		remove {
			set p $::dronebl_pub::permissions(remove)
			if {![matchattr $hand $p|$p $chan]} {
				puthelp "PRIVMSG $chan :$nick: You don't have access to that command.  If you normally do have access, try identing."
			} else {
				puthelp "PRIVMSG $chan :syntax: \002!$match id\002 \[id \[id \[id...\]\]\]"
				puthelp "PRIVMSG $chan :...where id is the index of the listing to remove.  Look up the id using the \002[lindex $::dronebl_pub::triggers(lookup) 0]\002 command."
			}
		}
		classes {
			set p $::dronebl_pub::permissions(classes)
			if {![matchattr $hand $p|$p $chan]} {
				puthelp "PRIVMSG $chan :$nick: You don't have access to that command.  If you normally do have access, try identing."
			} else {
				puthelp "PRIVMSG $chan :syntax: \002!$match\002 \[classid\]"
				puthelp "PRIVMSG $chan :... defaults to showing all classes unless \[classid\] is specified."
			}
		}
		help {
			set p $::dronebl_pub::permissions(help)
			if {![matchattr $hand $p|$p $chan]} {
				puthelp "PRIVMSG $chan :$nick: You don't have access to that command.  If you normally do have access, try identing."
			} else {
				puthelp "PRIVMSG $chan :syntax: \002!$match command\002"
				puthelp "PRIVMSG $chan :...where command is the trigger about which you would like more information.  Try \002[lindex $::dronebl_pub::triggers(showaccess) 0]\002 to see what is available to you."
			}
		}
		showaccess {
			set p $::dronebl_pub::permissions(showaccess)
			if {![matchattr $hand $p|$p $chan]} {
				puthelp "PRIVMSG $chan :$nick: You don't have access to that command.  If you normally do have access, try identing."
			} else {
				puthelp "PRIVMSG $chan :syntax: \002!$match\002"
				puthelp "PRIVMSG $chan :... shows the commands to which you currently have access.  If I don't recognize you on your current user@host, you might not have as much access as you should.  If that's the case, try identing."
			}
		}
		stfu {
			set p $::dronebl_pub::permissions(stfu)
			if {![matchattr $hand $p|$p $chan]} {
				puthelp "PRIVMSG $chan :$nick: You don't have access to that command.  If you normally do have access, try identing."
			} else {
				puthelp "PRIVMSG $chan :syntax: \002!$match\002"
				puthelp "PRIVMSG $chan :...makes me stop whatever the hell I'm doing and be quiet."
			}
		}
		*trigger* {
			set p $::dronebl_pub::permissions(help)
			if {![matchattr $hand $p|$p $chan]} {
				puthelp "PRIVMSG $chan :$nick: You don't have access to that command.  If you normally do have access, try identing."
			} else {
				puthelp "PRIVMSG $chan :$nick:  When I said to try [lindex $::dronebl_pub::triggers(help) 0] trigger, I didn't mean the actual word \"trigger.\"  Dumbass."
			}
		}
		default { puthelp "PRIVMSG $chan :$nick: DroneBL interface help.  Try \002[lindex $::dronebl_pub::triggers(showaccess) 0]\002 to see the triggers you can use, and \002[lindex $::dronebl_pub::triggers(help) 0] trigger\002 for more information." }
	}
}

proc showaccess {nick uhost hand chan txt} {
	global ::dronebl_pub::chanset_flag ::dronebl_pub::permissions ::dronebl_pub::triggers
	if {$chan != $nick && ![channel get $chan $::dronebl_pub::chanset_flag]} {return}
	set p $::dronebl_pub::permissions(showaccess)
	if {![matchattr $hand $p|$p $chan]} {
		puthelp "PRIVMSG $chan :$nick: Permission denied.  If you feel you have reached this message in error, try identing."
		return
	}
	set commands {}
	foreach command {add lookup remove classes help showaccess stfu} {
		if {[matchattr $hand $::dronebl_pub::permissions($command)]} {
			lappend commands [lindex $triggers($command) 0]
		}
	}
	puthelp "PRIVMSG $chan :$nick: You can use the following commands: [::dronebl::args2text [join $commands]]"
	return
}

proc stfu {nick uhost hand chan txt} {
	global ::dronebl_pub::chanset_flag ::dronebl_pub::permissions
	if {$chan != $nick && ![channel get $chan $::dronebl_pub::chanset_flag]} {return}
	set p $::dronebl_pub::permissions(stfu)
	if {[matchattr $hand $p|$p $chan]} {
		clearqueue all
		putserv "PRIVMSG $chan :$nick: Sorry.  Shutting up now."
	}
}

}; # end namespace eval

putlog "dronebl-pub.tcl loaded."
